import pathlib

from prestoplot import seeds, storages, story

PATH = pathlib.Path(__file__).parent

DATA = PATH / "data"


def test_it_should_generate_a_stable_story_with_seed():
    storage = storages.FileStorage(DATA)
    result = story.render_story(storage, "characters", seed="testing")
    expected = (
        "Our hero, Owenry, is a flashing-eyed, erring explorer, "
        "subjected to adverse conditions. He carries a whip."
    )
    assert result == expected

    result = story.render_story(storage, "characters", seed="testing")
    expected = (
        "Our hero, Owenry, is a flashing-eyed, erring explorer, "
        "subjected to adverse conditions. He carries a whip."
    )
    assert result == expected


def test_it_should_generate_a_stable_story_with_seeded_rng():
    storage = storages.FileStorage(DATA)
    rng = seeds.get_rng("foo")
    result = story.render_story(storage, "characters", seed=rng)
    expected = (
        "Our hero, Carter, is a lithe, married scientist, subjected to "
        "adverse conditions. He carries a whip."
    )
    assert result == expected

    result = story.render_story(storage, "characters", seed=rng)
    expected = (
        "Our heroine, Avery, is a superbly fit, rifle-wielding millionaire, "
        "swayed by pretense. She carries a rifle."
    )
    assert result == expected


def test_render_story_jinja2():
    storage = storages.FileStorage(DATA)
    result = story.render_story(storage, "characters_jinja", seed="testing")
    expected = (
        "Our hero, Noah, is a flashing-eyed, erring explorer, "
        "subjected to adverse conditions. He carries a whip."
    )
    assert result == expected



import contextlib

from . import seeds


def get_context(seed, **kwargs):
    context = {**kwargs}
    seeds.set_seed(context, seed)
    return context


@contextlib.contextmanager
def update_context(ctx, **kwargs):
    old_ctx = {key: ctx.get(key) for key in kwargs}
    ctx.update(kwargs)
    yield ctx
    ctx.update(old_ctx)



import hashlib
import random


def make_seed(rng=random):
    return hashlib.md5(str(rng.random()).encode("utf-8")).hexdigest()


def set_seed(context, seed):
    if seed is None or seed is random:
        seed = make_seed()
    elif isinstance(seed, random.Random):
        seed = make_seed(seed)

    context["seed"] = seed


def get_seed(context):
    try:
        seed = context["seed"]
    except KeyError:
        seed = context["seed"] = make_seed()

    if seed is random or isinstance(seed, random.Random):
        seed = context["seed"] = make_seed(seed)

    key = context.get("key")
    if key is not None:
        seed = f"{seed}-{key}"

    return seed


def get_rng(seed=None):
    if seed is None:
        return random
    elif isinstance(seed, random.Random):
        return seed
    else:
        return random.Random(seed)



import logging

from . import contexts, grammars


def render_story(storage, name, start="Begin", seed=None, **kwargs):
    logging.debug(f"Rendering {name} w/ {storage}")

    ctx = contexts.get_context(seed, **kwargs)
    ctx = grammars.parse_grammar_file(storage, name, ctx)
    return str(ctx[start])
